#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com")]
    public class SwingMomentum : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int strength = 10; // Default setting for Strength
            private int momentumLength = 5; // Default setting for MomentumLength
        // User defined variables (add any user defined variables below)
		private SwingOscillator so;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Bar, "Velocity"));
            Overlay				= false;
        }
		
		protected override void OnStartUp()
		{
			so=SwingOscillator(strength);
		}

        protected override void OnBarUpdate()
        {
			if (CurrentBar<momentumLength)
				return;
			
            Velocity.Set(so[0]-so[momentumLength]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Velocity
        {
            get { return Values[0]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Strength
        {
            get { return strength; }
            set { strength = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int MomentumLength
        {
            get { return momentumLength; }
            set { momentumLength = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SwingMomentum[] cacheSwingMomentum = null;

        private static SwingMomentum checkSwingMomentum = new SwingMomentum();

        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public SwingMomentum SwingMomentum(int momentumLength, int strength)
        {
            return SwingMomentum(Input, momentumLength, strength);
        }

        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public SwingMomentum SwingMomentum(Data.IDataSeries input, int momentumLength, int strength)
        {
            if (cacheSwingMomentum != null)
                for (int idx = 0; idx < cacheSwingMomentum.Length; idx++)
                    if (cacheSwingMomentum[idx].MomentumLength == momentumLength && cacheSwingMomentum[idx].Strength == strength && cacheSwingMomentum[idx].EqualsInput(input))
                        return cacheSwingMomentum[idx];

            lock (checkSwingMomentum)
            {
                checkSwingMomentum.MomentumLength = momentumLength;
                momentumLength = checkSwingMomentum.MomentumLength;
                checkSwingMomentum.Strength = strength;
                strength = checkSwingMomentum.Strength;

                if (cacheSwingMomentum != null)
                    for (int idx = 0; idx < cacheSwingMomentum.Length; idx++)
                        if (cacheSwingMomentum[idx].MomentumLength == momentumLength && cacheSwingMomentum[idx].Strength == strength && cacheSwingMomentum[idx].EqualsInput(input))
                            return cacheSwingMomentum[idx];

                SwingMomentum indicator = new SwingMomentum();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.MomentumLength = momentumLength;
                indicator.Strength = strength;
                Indicators.Add(indicator);
                indicator.SetUp();

                SwingMomentum[] tmp = new SwingMomentum[cacheSwingMomentum == null ? 1 : cacheSwingMomentum.Length + 1];
                if (cacheSwingMomentum != null)
                    cacheSwingMomentum.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSwingMomentum = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SwingMomentum SwingMomentum(int momentumLength, int strength)
        {
            return _indicator.SwingMomentum(Input, momentumLength, strength);
        }

        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SwingMomentum SwingMomentum(Data.IDataSeries input, int momentumLength, int strength)
        {
            return _indicator.SwingMomentum(input, momentumLength, strength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SwingMomentum SwingMomentum(int momentumLength, int strength)
        {
            return _indicator.SwingMomentum(Input, momentumLength, strength);
        }

        /// <summary>
        /// Calculate Momentum in Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SwingMomentum SwingMomentum(Data.IDataSeries input, int momentumLength, int strength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SwingMomentum(input, momentumLength, strength);
        }
    }
}
#endregion
